@@ -6,29 +6,31 @@ utils::globalVariables(c(
 #' Comprehensive Multivariate Normality and Diagnostic Function
 #'
 #' Conduct multivariate normality tests, outlier detection, univariate normality tests,
-#' descriptive statistics, and Box-Cox transformation in one wrapper.
+#' descriptive statistics, and Box-Cox or Yeo-Johnson transformation in one wrapper.
 #'
-#' @param data A numeric matrix or data frame (rows = observations, columns = variables).
-#' @param subset Optional character; name of a grouping variable in \code{data} for subset analyses.
-#' @param mvn_test Character; one of \code{"mardia"}, \code{"hz"}, \code{"royston"}, 
-#'   \code{"doornik_hansen"}, or \code{"energy"}. Default: \code{"hz"}.
-#' @param use_population Logical; if \code{TRUE}, uses the population covariance estimator \eqn{\frac{n-1}{n} \times \Sigma}; otherwise uses the sample covariance. Default is \code{TRUE}.
-#' @param tol Numeric; tolerance for matrix inversion via \code{solve()}. Default: \code{1e-25}.
-#' @param alpha Numeric; significance level for ARW outlier cutoff when 
-#'   \code{multivariate_outlier_method = "adj"}. Default: \code{0.05}.
-#' @param scale Logical; if \code{TRUE}, standardizes the data before analysis. Default: \code{FALSE}.
-#' @param descriptives Logical; if \code{TRUE}, compute descriptive statistics. Default: \code{TRUE}.
-#' @param transform Character; one of \code{"none"}, \code{"log"}, \code{"sqrt"}, \code{"square"}.
-#'   Applies marginal transformations before analysis. Default: \code{"none"}.
-#' @param box_cox_transform Logical; if \code{TRUE}, applies Box-Cox transformation to all variables. Default: \code{FALSE}.
-#' @param box_cox_transform_type Character; either \code{"optimal"} or \code{"rounded"} lambda for Box-Cox. Default: \code{"optimal"}.
-#' @param R Integer; number of bootstrap replicates for \code{"energy"} test. Default: \code{1000}.
-#' @param univariate_test Character; one of \code{"SW"}, \code{"CVM"}, \code{"Lillie"}, 
-#'   \code{"SF"}, \code{"AD"}. Default: \code{"AD"}.
-#' @param multivariate_outlier_method Character; \code{"none"}, \code{"quan"}, or \code{"adj"}.
-#'   Default: \code{"none"}.
-#' @param show_new_data Logical; if \code{TRUE}, include cleaned data (non-outliers). Default: \code{FALSE}.
-#' @param tidy Logical; if \code{TRUE}, returns the results as a tidy data frame with an added \code{Group} column. Default is \code{TRUE}.
+#' @param data A numeric matrix or data frame where each row represents an observation and each column represents a variable. All variables should be numeric; non-numeric columns will be ignored or cause an error depending on implementation.
+#' @param subset Optional character string indicating the name of a grouping variable within the data. When provided, analyses will be performed separately for each level of the grouping variable. This is useful for comparing multivariate normality or outlier structure across groups.
+#' @param mvn_test A character string specifying which multivariate normality test to use. Supported options include "mardia" (Mardia's test), "hz" (Henze-Zirkler's test), "hw" (Henze-Wagner's test), "royston" (Royston's test), "doornik_hansen" (Doornik-Hansen test), and "energy" (Energy-based test). The default is "hz", which provides good power for detecting departures from multivariate normality.
+#' @param use_population A logical value indicating whether to use the population version of the covariance matrix estimator. If TRUE, scales the covariance matrix by (n - 1)/n to estimate the population covariance. If FALSE, the sample covariance matrix is used instead. The default is TRUE.
+#' @param tol A small numeric value used as the tolerance parameter for matrix inversion via solve(). This is important when working with nearly singular covariance matrices. The default value is 1e-25, which ensures numerical stability during matrix computations.
+#' @param alpha A numeric value specifying the significance level used for defining outliers when the multivariate outlier detection method is set to "adj" (adjusted robust weights). This threshold controls the false positive rate for identifying multivariate outliers. The default is 0.05.
+#' @param scale A logical value. If TRUE, the input data will be standardized (zero mean and unit variance) before analysis. This is typically recommended when variables are on different scales. Default is FALSE.
+#' @param descriptives A logical value indicating whether to compute descriptive statistics (mean, standard deviation, skewness, and kurtosis) for each variable before conducting multivariate normality or outlier analyses. Default is TRUE.
+#' @param transform A character string specifying a marginal transformation to apply to each variable before analysis. Options are "none" (no transformation), "log" (natural logarithm), "sqrt" (square root), and "square" (square of the values). The default is "none".
+#' @param impute A character string specifying method for handling missing data. One of \code{"none"}, \code{"mean"}, \code{"median"}, or \code{"mice"}. Default: \code{"none"}.
+#' @param power_family A character string specifying the type of power transformation family to apply before analysis. Options include "none" (no transformation), "bcPower" (Box-Cox transformation for positive data), "bcnPower" (Box-Cox transformation that allows for negatives), and "yjPower" (Yeo-Johnson transformation for real-valued data). Default is "none".
+#' @param power_transform_type A character string indicating whether to use the "optimal" or "rounded" lambda value for the selected power transformation. "optimal" uses the estimated value with maximum likelihood, while "rounded" uses the closest integer value for interpretability. Default is "optimal".
+#' @param bootstrap Logical; if \code{TRUE}, p-values for Mardia, Henze-Zirkler
+#'   and Royston tests are obtained via bootstrap resampling. Default is
+#'   \code{FALSE}.
+#' @param B Integer; number of bootstrap replicates used when
+#'   \code{bootstrap = TRUE} or \code{mvn_test = "energy"}. Default is \code{1000}.
+#' @param cores Integer; number of cores to use for bootstrap computation.
+#'   Default is 1.
+#' @param univariate_test A character string indicating which univariate normality test to apply to individual variables when such summaries are requested. Options include "SW" (Shapiro-Wilk), "CVM" (Cramér–von Mises), "Lillie" (Lilliefors/Kolmogorov-Smirnov), "SF" (Shapiro–Francia), and "AD" (Anderson–Darling). Default is "AD".
+#' @param multivariate_outlier_method A character string that specifies the method used for detecting multivariate outliers. Options are "none" (no outlier detection), "quan" (robust Mahalanobis distance based on quantile cutoff), and "adj" (adjusted robust weights with a significance threshold). Default is "none".
+#' @param show_new_data A logical value. If TRUE, the cleaned data with identified outliers removed will be included in the output. This is useful for downstream analysis after excluding extreme observations. Default is FALSE.
+#' @param tidy A logical value. If TRUE, the output will be returned as a tidy data frame, making it easier to use with packages from the tidyverse. A "Group" column will be included when subset analysis is performed. Default is TRUE.
 
 #' @return A named list containing:
 #' \describe{
@@ -38,7 +40,7 @@ utils::globalVariables(c(
 #'   \item{descriptives}{(Optional) A data frame of descriptive statistics if \code{descriptives = TRUE}.}
 #'   \item{multivariate_outliers}{(Optional) A data frame of flagged multivariate outliers if \code{multivariate_outlier_method != "none"}.}
 #'   \item{new_data}{(Optional) Original data with multivariate outliers removed if \code{show_new_data = TRUE}.}
-#'   \item{box_cox_lambda}{(Optional) Estimated Box-Cox lambda values if \code{box_cox_transform = TRUE}.}
+#'   \item{powerTransformLambda}{(Optional) Estimated power transform lambda values if \code{power_family = "bcPower"}.}
 #'   \item{data}{The processed data matrix used in the analysis (transformed and/or cleaned).}
 #'   \item{subset}{(Optional) The grouping variable used for subset analysis, if applicable.}
 #' }
@@ -50,10 +52,10 @@ utils::globalVariables(c(
 #' If there are missing values in the data, a listwise deletion will be applied and a complete-case analysis will be performed.
 #'
 #' If \code{mvn_test = "hz"}, it calculates the Henze-Zirkler's multivariate normality test. The Henze-Zirkler test is based on a non-negative functional distance that measures the distance between two distribution functions. If the data is multivariate normal, the test statistic HZ is approximately lognormally distributed. It proceeds to calculate the mean, variance and smoothness parameter. Then, mean and variance are lognormalized and the p-value is estimated.
-#' If there are missing values in the data, a listwise deletion will be applied and a complete-case analysis will be performed.
+#'
+#' If \code{mvn_test = "hw"}, it calculates the Henze-Wagner's multivariate normality test. The Henze-Wagner test is based on a class of weighted L2-statistics that quantify the deviation of the empirical characteristic function from that of the multivariate normal distribution. It uses a weight function involving a smoothness parameter to control the influence of differences in the tails. The test statistic is computed and its null distribution is approximated to obtain the p-value.
 #'
 #' If \code{mvn_test = "royston"}, it calculates the Royston's multivariate normality test. A function to generate the Shapiro-Wilk's W statistic needed to feed the Royston's H test for multivariate normality However, if kurtosis of the data greater than 3 then Shapiro-Francia test is used for leptokurtic samples else Shapiro-Wilk test is used for platykurtic samples.
-#' If there are missing values in the data, a listwise deletion will be applied and a complete-case analysis will be performed. Do not apply Royston's test, if dataset includes more than 5000 cases or less than 3 cases, since it depends on Shapiro-Wilk's test.
 #'
 #' If \code{mvn_test = "doornik_hansen"}, it calculates the Doornik-Hansen's multivariate normality test. The code is adapted from asbio package (Aho, 2017).
 #'
@@ -64,7 +66,7 @@ utils::globalVariables(c(
 #' @references
 #' Korkmaz S, Goksuluk D, Zararsiz G. MVN: An R Package for Assessing Multivariate Normality. The R Journal. 2014 6(2):151-162. URL \url{https://journal.r-project.org/archive/2014-2/korkmaz-goksuluk-zararsiz.pdf}
 #'
-#' Mardia, K. V. (1970), Measures of multivariate skewnees and kurtosis with applications. Biometrika, 57(3):519-530.
+#' Mardia, K. V. (1970), Measures of multivariate skewness and kurtosis with applications. Biometrika, 57(3):519-530.
 #'
 #' Mardia, K. V. (1974), Applications of some measures of multivariate skewness and kurtosis for testing normality and robustness studies. Sankhy A, 36:115-128.
 #'
@@ -135,11 +137,14 @@ mvn <- function(data,
                 scale = FALSE,
                 descriptives = TRUE,
                 transform = "none",
-                R = 1000,
+                impute = "none",
+                bootstrap = FALSE,
+                B = 1000,
+                cores = 1,
                 univariate_test = "AD",
                 multivariate_outlier_method = "none",
-                box_cox_transform = FALSE,
-                box_cox_transform_type = "optimal",
+                power_family = "none",
+                power_transform_type = "optimal",
                 show_new_data = FALSE,
                 tidy = TRUE) {
   
@@ -158,37 +163,52 @@ mvn <- function(data,
     stop("The 'subset' variable does not exist in the data.")
   }
   
-  
-  if (box_cox_transform && transform != "none") {
-    stop("Use either `transform` or `box_cox_transform`, not both.")
+  p_tmp <- if (!is.null(subset)) ncol(data) - 1 else ncol(data)
+  n_tmp <- nrow(data)
+  if (p_tmp > n_tmp && mvn_test != "hw") {
+    warning("Number of variables exceeds sample size; consider mvn_test = 'hw'.")
   }
   
-  if (!mvn_test %in% c("mardia", "hz", "royston", "doornik_hansen", "energy")) {
-    stop("Invalid mvn_test. Choose from: 'mardia', 'hz', 'royston', 'doornik_hansen', 'energy'.")
+  power_family <- match.arg(power_family, c("none", "bcPower", "bcnPower", "bcnPowerInverse", "yjPower", "basicPower"))
+  power_transform_type <- match.arg(power_transform_type, c("optimal", "rounded"))
+  impute <- match.arg(impute, c("none", "mean", "median", "mice"))
+  
+
+  if (power_family != "none" && transform != "none") {
+    stop("Use either `transform` or `power_family`, not both.")
+  }
+  
+  if (!mvn_test %in% c("mardia", "hz", "hw", "royston", "doornik_hansen", "energy")) {
+    stop("Invalid mvn_test. Choose from: 'mardia', 'hz', 'hw', 'royston', 'doornik_hansen', 'energy'.")
   }
   
   if (!univariate_test %in% c("SW", "CVM", "Lillie", "SF", "AD")) {
     stop("Invalid univariate_test. Choose from: 'SW', 'CVM', 'Lillie', 'SF', 'AD'.")
   }
   
-  if (box_cox_transform && any(data <= 0)) {
+  if (power_family == "bcPower" && any(data[sapply(data, is.numeric)] <= 0)) {
     stop("Box-Cox transformation requires strictly positive data.")
   }
   
   
   if (anyNA(data)) {
-    warning("Missing values detected. Rows with missing data will be removed.")
-    data <- data[complete.cases(data), ]
+    if (impute == "none") {
+      removed_rows <- sum(!complete.cases(data))
+      warning(sprintf("Missing values detected in %d rows. These rows will be removed.", removed_rows))
+      data <- data[complete.cases(data), ]
+    } else {
+      message(sprintf("Missing values detected. Applying '%s' imputation method.", impute))
+      data <- impute_missing(data, method = impute)
+    }
   }
   
-  
   if (is.null(subset)) {
-    if (box_cox_transform) {
-      result = box_cox_transform(data, type = box_cox_transform_type)
+    if (power_family != "none") {
+      result = power_transform(data, family = power_family, type = power_transform_type)
       data = result$data
-      BoxCoxPower = result$lambda
-      
-    }
+      powerTransformLambda = result$lambda
+
+    } 
     
     if (transform == "log") {
       data = apply(data, 2, log)
@@ -211,28 +231,63 @@ mvn <- function(data,
     
     if (!(dim(data)[2] < 2 || is.null(dim(data)))) {
       if (mvn_test == "mardia") {
-        mvnResult = mardia(data, use_population = use_population, tol = tol)
+        mvnResult = mardia(
+          data,
+          use_population = use_population,
+          tol = tol,
+          bootstrap = bootstrap,
+          B = B,
+          cores = cores
+        )
         
       }
       
       if (mvn_test == "hz") {
-        mvnResult = hz(data, use_population = use_population, tol = tol)
+        mvnResult = hz(
+          data,
+          use_population = use_population,
+          tol = tol,
+          bootstrap = bootstrap,
+          B = B,
+          cores = cores
+        )
+        
+      }
+      
+      if (mvn_test == "hw") {
+        mvnResult = hw(
+          data,
+          use_population = use_population,
+          tol = tol,
+          bootstrap = bootstrap,
+          B = B,
+          cores = cores
+        )
         
       }
       
       if (mvn_test == "royston") {
-        mvnResult = royston(data, tol = tol)
+        mvnResult = royston(
+          data,
+          tol = tol,
+          bootstrap = bootstrap,
+          B = B,
+          cores = cores
+        )
         
       }
       
       
       if (mvn_test == "doornik_hansen") {
-        mvnResult = doornik_hansen(data)
+        mvnResult = doornik_hansen(data,
+                                   bootstrap = bootstrap,
+                                   B = B,
+                                   cores = cores)
         
       }
       
       if (mvn_test == "energy") {
-        mvnResult = energy(data, R = R)
+        mvnResult = energy(data, B = B)
         
       }
       
@@ -292,7 +347,7 @@ mvn <- function(data,
 
     
   } else{
-    if (box_cox_transform) {
+    if (power_family != "none") {
       sData = split(data[, !(colnames(data) %in% subset)], data[, subset])
       comp <- lapply(sData, complete.cases)
       clean_data <- Map(function(d, c) {
@@ -300,25 +355,24 @@ mvn <- function(data,
       }, sData, comp)
       sData <- lapply(sData, function(x)
         x[complete.cases(x), ])
-      
-      result = lapply(sData, box_cox_transform, type = box_cox_transform_type)
+
+      result = lapply(sData, power_transform, family = power_family, type = power_transform_type)
       dataList = list()
-      bcList = list()
-      
+      powerTransformList = list()
+
       for (i in 1:length(result)) {
         dataList[[i]] = result[[i]][[1]]
-        bcList[[i]] = result[[i]][[2]]
+        powerTransformList[[i]] = result[[i]][[2]]
       }
-      
+
       data = cbind.data.frame(do.call(rbind.data.frame, dataList), data[subset][complete.cases(data), ])
-      
+
       colnames(data) = colnms
-      
-      
-      BoxCoxPower = bcList
-      names(BoxCoxPower) = names(sData)
-      
-    }
+
+      powerTransformLambda = powerTransformList
+      names(powerTransformLambda) = names(sData)
+
+    } 
     
     if (transform == "log") {
       tData = apply(data[!(colnames(data) %in% subset)], 2, log)
@@ -354,33 +408,68 @@ mvn <- function(data,
     
     if (!(is.null(lapply(splitData, dim)[[1]]))) {
       if (mvn_test == "mardia") {
-        mvnResult = lapply(splitData,
-                           mardia,
-                           use_population = use_population,
-                           tol = tol)
+        mvnResult = lapply(
+          splitData,
+          mardia,
+          use_population = use_population,
+          tol = tol,
+          bootstrap = bootstrap,
+          B = B,
+          cores = cores
+        )
         
       }
       
       if (mvn_test == "hz") {
-        mvnResult = lapply(splitData,
-                           hz,
-                           use_population = use_population,
-                           tol = tol)
-        
+        mvnResult = lapply(
+          splitData,
+          hz,
+          use_population = use_population,
+          tol = tol,
+          bootstrap = bootstrap,
+          B = B,
+          cores = cores
+        )
+      }
+      
+      
+      if (mvn_test == "hw") {
+        mvnResult = lapply(
+          splitData,
+          hw,
+          use_population = use_population,
+          tol = tol,
+          bootstrap = bootstrap,
+          B = B,
+          cores = cores
+        )
       }
       
       if (mvn_test == "royston") {
-        mvnResult = lapply(splitData, royston, tol = tol)
+        mvnResult = lapply(
+          splitData,
+          royston,
+          tol = tol,
+          bootstrap = bootstrap,
+          B = B,
+          cores = cores
+        )
         
       }
       
       if (mvn_test == "doornik_hansen") {
-        mvnResult = lapply(splitData, doornik_hansen)
+        mvnResult = lapply(
+          splitData,
+          doornik_hansen,
+          bootstrap = bootstrap,
+          B = B,
+          cores = cores
+        )
         
       }
       
       if (mvn_test == "energy") {
-        mvnResult = lapply(splitData, energy, R = R)
+        mvnResult = lapply(splitData, energy, B = B)
         
       }
       
@@ -612,10 +701,10 @@ mvn <- function(data,
     
   }
   
-  if (box_cox_transform) {
-    result = c(result, list(box_cox_lambda = BoxCoxPower))
-    
-  }
+  if (power_family != "none") {
+    result = c(result, list(power_transform_lambda = powerTransformLambda))
+
+  } 
   
   result = c(result, list(data = data, subset = subset))
   
